#!/usr/bin/perl
#

use strict;
use warnings;
use Socket;
use Sys::Hostname;
use lib '/opt/novell/iprint/lib/perl5';
use Net::Domain qw(hostfqdn);
use File::Spec qw(tmpdir);
use File::Temp qw(mktemp tempdir);
use Net::LDAP::Util qw(ldap_explode_dn escape_dn_value);
use Data::Dumper;
use Getopt::Long qw(:config no_ignore_case bundling);
use XML::Simple;
use IO::File;
use IPC::Open3 qw(open3);
use URI;
use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
use POSIX qw(EINTR EAGAIN);
use Cwd qw(getcwd);
require LWP::UserAgent;
use MIME::Base64 qw(encode_base64);
use Term::ReadKey qw(ReadMode ReadLine);
#
use Net::LDAP;
use Net::LDAPS;
use Net::LDAP::LDIF;
use File::Path qw(rmtree);
use File::Basename qw(basename);
#Constants
use constant _IPRINTMIG      => 'iprintmig';
use constant _VERSION        => '0.1';
use constant _COPYRIGHT      => 'Copyright (c) 2007 Novell Inc.';
use constant _CONTENT        => '_xml_text_content';
#################################
#Globals
my %opts;
my $xmlref;
my $tmpdir;
my $xml_out_fh = new IO::File;
my %drivers;
my %ipmap;
my $xml_out;
my @objects_to_cleanup;
my $src_tree_name;

###################################################################
sub ParseOpts
{
	my ($spfd, $dpfd, $pfd, $pf, );
	my @printers;

	#Defaults
	eval
	{
		GetOptions("h|help" => \$opts{'help'},
			   "v|verbose+" => \$opts{'verbose'},
			   "s|src=s" => \$opts{'src'},
			   "u|src-user=s" => \$opts{'src-user'},
			   "p|src-pass=s" => \$opts{'src-pass'},
			   "f|cleanup-file=s" => \$pf,
			   "c|base-dn=s" => \$opts{'base-dn'},
			   "ssl" => \$opts{'ssl'},
			   "port=s" => \$opts{'port'}) or BasicUsage('');
	   };
	   Usage("Couldn't parse options: $@\n") if $@;
	 	  

	if ($pf)
	   {
		   if (open(PRINTERS, '<', $pf))
		   {
			   while (<PRINTERS>)
			   {
				   chomp;
				   chop  if (/\r$/);	
				   push(@objects_to_cleanup, $_);
			   }
			   close(PRINTERS);
		   }
		   else
		   {
			   print "Unable to get printer list from file $pf: $!";
			   print "\n";
				exit;
		   }
	   }
	my @cmd1 = "ndsstat -h $opts{'src'}";
	my($cmdout, $cmderr, $exitcode) = ('', '', undef);
	my $pid;
	my($wdr,$rdr,$err);
	$pid = open3($wdr, $rdr,$err, @cmd1);
	my @outlines1 = <$rdr>;
	foreach my $line1 (@outlines1) 
	{
    		chomp($line1);
		if($line1 =~ s/^Tree Name: //)
		{
			$src_tree_name=$line1;
		}
	}
	close($rdr);


}
sub Usage(;$)
{
	my $err = 0;
	if (@_)
	{
		my $warning = shift;
		print {*STDERR} "$warning\n" if $warning;
		$err = 1;
	}
	print {$err ? *STDERR : *STDOUT} <<EOF;
	Usage: $0 -s <source server> -u <source user(dn format)> --ssl --port <LDAP Port> -f <filename>  
Options:
-h | --help             Print this summary
-s | --src <server>     Source server ip address
-u | --src-user <user>  dn-style admin user DN for source server (e.g. cn=admin,o=novell)
-p | --src-pass <pass>  Password of source server admin user
-f | --renamed-printers-file <filename(/etc/opt/novell/iprint/conf/renamed_printer_objects) >
--ssl                   If ssl enabled on the server.
--port                  LDAP Enabled Port.
EOF
exit($err);
}



sub BasicUsage(;$)
{
	my $err = 0;
	if (@_)
	{
		my $warning = shift;
		print {*STDERR} "$warning\n" if $warning;
		$err = 1;
	}
	print {$err ? *STDERR : *STDOUT} <<EOF;
	Usage: $0 -s <source server> -u <source user(dn format)> -p <source password> --port -f <filename>  
EOF
exit($err);
}
sub LdapToDN
{
	my @usr = split(/,/,$opts{'src-user'});
	my @dns;
	foreach my $arr (@usr)
	{
		my @v = split(/=/,$arr);
		push (@dns,$v[1]);
	}
	my $dn = join('.',@dns);
	return $dn;
}
#To get the source xml file
sub cleanupObject
{
	my $BASEDN   = $opts{'base-dn'};#"o=novell";
	#my $ADMIN = UserNameToDNS($opts{'src-user-dns'});#"cn=admin,o=novell";
	my $ADMIN = $opts{'src-user'};#"cn=admin,o=novell";
	my $tag = " ";
	my $dname = " ";
	my $src_dn_name = LdapToDN();
	my $ldap;
	my $count;
	my $r;
	if ($opts{'ssl'})
	{
			`ADM_PWD=$opts{'src-pass'} /opt/novell/oes-install/util/getSSCert -a $opts{'src'}  -t $src_tree_name -u $src_dn_name`;
			`openssl x509 -inform der -in /etc/opt/novell/certs/SSCert.der -out /tmp/SSCert.pem`;
			until($ldap = Net::LDAPS->new($opts{'src'}, port => $opts{'port'}, verify => 'require', cafile => "/tmp/SSCert.pem" ))
 			{
				die "Can not connect to ldap://$opts{'src'}:$opts{'port'}/" if ++$count > 10;
  				sleep 1;
			}
	}
	else
	{
		until($ldap = Net::LDAP->new($opts{'src'}, port => $opts{'port'})) {
  		die "Can not connect to ldap://$opts{'src'}:$opts{'port'}/" if ++$count > 10;
		sleep 1;
		}
	}
	$r = $ldap->bind($ADMIN, password => $opts{'src-pass'}, version=>3);
	die $r->error if $r->code;

	foreach $dname (@objects_to_cleanup)
	{
		print "Cleaning the Printer Object: $dname\n\n";
		#connect to the server
		#MigError->new("", $r->error, MigError::ERROR)->handle() if $r->code;
		$r = $ldap->delete("$dname");
		if ( $r->code )
		{
			my $msg = $r->error;
			print "Failed to delete $dname with Error $msg\n\n";
		}
		#die $r->error if $r->code;
		#MigError->new("", $r->error, MigError::WARNING)->handle() if $r->code;
		
	}
		$ldap->unbind;
}

  if( $#ARGV == -1 || ($#ARGV < 7) || ($#ARGV > 8) )
  {
       	Usage();
  }

ParseOpts;
Usage if $opts{'help'};
if ( !$opts{'src-pass'} )
{
	print "Enter Source Server Password: ";
	ReadMode('noecho');
	chomp($opts{'src-pass'} = ReadLine(0));
	ReadMode(0);
	print "\n";
}
cleanupObject();

