#!/usr/bin/perl
#!C:/Perl/bin/perl

#/+==========================================================================+
# Copyright (c) 1994 - 2009 Unpublished Work of Novell, Inc.  All rights reserved.
#//|
#//| THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL, PROPRIETARY
#//| AND TRADE SECRET INFORMATION OF NOVELL, INC.
#//|
#//| ACCESS TO THIS WORK IS RESTRICTED TO (I) NOVELL EMPLOYEES WHO HAVE A
#//| NEED TO KNOW HOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR ASSIGNMENTS
#//| AND (II) ENTITIES OTHER THAN NOVELL, INC. WHO HAVE ENTERED INTO
#//| APPROPRIATE LICENSE AGREEMENTS.
#//|
#//| NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED,
#//| DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED,
#//| EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED
#//| WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC.
#//|
#//| ANY USE OR EXPLOITATION OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT
#//| THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#//+==========================================================================+
#//| Source module name: zcm-migrate-print-policy.pl
#//+==========================================================================+
#//|$Author:  $              $Locker:  $
#//|$Revision: 1.0 $            
#//|$Date: 2009/02/16 23:19:51 $
#//+===========================================================================


use strict;
use warnings;
use Getopt::Long qw(:config no_ignore_case bundling);

use MIME::Base64 qw(encode_base64);
use Data::Dumper;
use File::Copy;


#Globals
my %opts;
my ($HOST, $PASSWD, $CONF, $ADMIN, $SRC, $DEST, $FILENAME,$PORT,$XMLDIR);
my ($REMALL, $SERVER,$CHANGEDEFAULT, $LOGDIR, $LOGFILE);
my ($command_1, $command_3, $command_4, $command_5, $command_def, $ret, $tmpfile);

#Get the OS version
my $os = $^O;

$HOST = "127.1.0.0";
my $file ='';
#my $LOGFILE;
my $datetime;
if ($os =~ /MSWin32/)
{
$LOGDIR='C:/';
$LOGFILE='zcm10-migration.log';
}
else
{
$LOGDIR="/var/opt/novell/migration";
$LOGFILE="/var/opt/novell/migration/zcm10-migration.log";
}

my (@originalprinters, @migprinters);

#Main program

parse_opts();
get_policies();
cleanup();

#Function to read parameters
sub parse_opts
{	my %opts;
       my $val1=
       eval
        {
        	GetOptions("h|help" => \$opts{'help'},
                           "v|verbose+" => \$opts{'verbose'},
                           "a|admin=s" => \$opts{'admin'},
                           "p|passwd=s" => \$opts{'passwd'},
                           "P|port=i"=>\$opts{'port'},
                           "n|netware"=>\$opts{'netware'},
                           "l|linux"=>\$opts{'linux'},
                           "s|src=s" => \$opts{'src'},
                           "d|dest=s" => \$opts{'dest'},
                           "r|remove" => \$opts{'rem'},
                           "c|default=s" => \$opts{'default'},
                           "f|file=s" => \$opts{'file'}, 
                           "x|xml:s" => \$opts{'xml'}) or usage();
	};
    
	usage("Options could not be parsed: $@\n") if $@;
	usage() if $opts{'help'};
       
        #This takes all the compulsory variables from command line 
	$ADMIN = $opts{'admin'};
	$PASSWD = $opts{'passwd'};
        $SRC = $opts{'src'};
        $DEST = $opts{'dest'};
	
	#These are optional parameters
	$CONF = $opts{'file'};
	$XMLDIR = $opts{'xml'};
        $REMALL = $opts{'rem'};    
        $CHANGEDEFAULT = $opts{'default'};

	unless ((defined $SRC) && (defined $DEST) && (defined $ADMIN) && (defined $PASSWD))
      	{
        	print STDOUT "Arguments are not specified correctly.\n";
		usage();
        	exit 1;

        }
	if($SRC =~ /[a-z]/i)
	{
		warn("Warning: You have mentioned a DNS name instead of IP address.Policies will change if only they are configured with DNS name of the source server  \n");
	}

	updatelog("username $ADMIN");

       #The optional variables file takes the value ="renamed_printer_objects" if no filename is entered. 
       #renamed_printer_objects is a file which is generated as a output to the migration of NetWare to Linux
       if($opts{'file'}){ 
          $FILENAME =$opts{'file'};
       }else{
       $FILENAME = "renamed_printer_objects";
       } 
       
       #The port is optionally specified in the command line. If nothing specified default value of 80 is taken
       if($opts{'port'}){
        $PORT = $opts{'port'};  
       }else{
       $PORT = 80;
       }     

}
sub get_policies{
        my ($IN,$j);
        $j=0; 

	 updatelog("Opening the file that contains the name of the renamed printers");
         print STDOUT "Opening the file that contains the name of the renamed printers \n";
         
	if (!(-e "$FILENAME")) {
	print STDOUT "File renamed_printer_objects does not exist! Copy file to current directory from /opt/novell/migration/sbin of the target server \n";
	exit 1;
	}

	($FILENAME) ? open IN,"<$FILENAME" || die("Error : Couldn't open data file $FILENAME\n") : usage("Usage : File that contains the list of migrated printers does not exist or you do not have sufficient rights to open it.\n");
      
         updatelog("Getting the list of original printers and migrated printers");
         print STDOUT "Getting the list of original printers and migrated printers\n";
       
	while(<IN>)
	{
   	       my ($line)= $_;
               my (@printer_1,@printer_2,@printer_3);
       	       chomp($line);
	       @printer_1 = split(',',$line);
               chomp($printer_1[0]);
	       @printer_2 = split('=',$printer_1[0]);
               $originalprinters[$j] = $printer_2[1];
	       updatelog("Original printers:$originalprinters[$j] ");
	       my $str1 = substr($printer_2[1],-1);
	       if($str1 eq "L"){
	    		$migprinters[$j] =substr( $printer_2[1],0,length($printer_2[1])-2);
		}
		else{
               		$migprinters[$j] =substr( $printer_2[1],0,length($printer_2[1])-3);
		}	
	       updatelog("Migrated printers:$migprinters[$j] ");

		$j = $j + 1;
	}
	close(IN);
	
	updatelog("The original printer array and the migrated printer array are populated");
        print STDOUT "The original printer array and the migrated printer array are populated\n";

	usage("Data file does not contain any printer name\n") if !$j;

	if($XMLDIR)
	{
		migrate_all_policies();
	}
	else
	{
		migrate_policies();
	}
}

#Display help
sub usage(;$)

{

	my $err = 0;
        if (@_)
        {
                my $warning = shift;
                print {*STDERR} "$warning\n" if $warning;
                $err = 1;
        }
             my $prgname = $0;
	
        print {$err ? *STDERR : *STDOUT} <<EOF;
Usage: $prgname  -a <Administrator name> -p <Administrator password> -s <Source server> -d <Destination server> [options] 
Options:
-h | --help    Print this summary
-a | --admin   <Administrator>       
-p | --passwd  <Administrator password>
-P | --port    <Port>
-n | --netware <OS type Netware>
-l | --linux   <OS type Linux>
-s | --src     <Source NetWare server>
-d | --dest    <Destination Linux server>   
-r | --rem     Deletes the old policies
-c | --change  Change default
-f | --file    <File name> Specify the file name which has the list of printers to be migrated
-x | --xml     <Directory name> Directory containing  the policies in xml form
EOF
exit($err);
}

#
sub migrate_policies
{       
        my $j=0;
        my $i=0;
        my $k=0;
        my $l=0;

       # Cloning all the zen policies in the source IPP server to the destination ipp server. This is done for migrated servers only
       updatelog("The new policies are getting created");   		
       print STDOUT "Please wait..\n";
       print STDOUT "The new policies are getting created.\n";

       # Changing default if the change default printer name is given
        if(defined $CHANGEDEFAULT){
          chomp($CHANGEDEFAULT);
          foreach(@migprinters){

          if($CHANGEDEFAULT eq $migprinters[$l]){
	  if ($os =~ /MSWin32/)
	  {
          $command_def = "ippman.bat clone -uri \"ipp:\/\/$SRC\/ipp\/$CHANGEDEFAULT\" -uri2 \"ipp:\/\/$DEST\/ipp\/$CHANGEDEFAULT\" -default true -updatedriver true -server $HOST -port $PORT -username $ADMIN -password $PASSWD";
	  }
	  else
	  {
          $command_def = "ippman.sh clone -uri \"ipp:\/\/$SRC\/ipp\/$CHANGEDEFAULT\" -uri2 \"ipp:\/\/$DEST\/ipp\/$CHANGEDEFAULT\" -default true -updatedriver true -server $HOST -port $PORT -username $ADMIN -password $PASSWD";
	  }	
          updatelog("Executing clone command from $SRC to $DEST for default printer $CHANGEDEFAULT");

	  #system ($command_def .">>$LOGFILE") ==0 or die("$datetime: Clone command is unsuccessful.Check log for more information");
          `$command_def`; 
       	  $ret = $? >> 8;
          if($ret )
          {
		updatelog("Clone command is unsuccessful.Check log for more information");	
		die("$datetime: Clone command is unsuccessful.Check log for more information");	
          }
          last;
         }
       $l++;
       }
         if($l > $#migprinters){
             print STDOUT "The default printer $CHANGEDEFAULT is not in your list of printers.\n";
        }
           }#end of if(CHANGEDEFAULT)


       foreach(@migprinters) { 
       if ($os =~ /MSWin32/)	       
       {
       $command_1 = "ippman.bat clone -uri \"ipp:\/\/$SRC\/ipp\/$migprinters[$j]\" -uri2 \"ipp:\/\/$DEST\/ipp\/$migprinters[$j]\"  -updatedriver true -server $HOST -port $PORT -username $ADMIN -password $PASSWD";
	}
	else
	{
       $command_1 = "ippman.sh clone -uri \"ipp:\/\/$SRC\/ipp\/$migprinters[$j]\" -uri2 \"ipp:\/\/$DEST\/ipp\/$migprinters[$j]\"  -updatedriver true -server $HOST -port $PORT -username $ADMIN -password $PASSWD";
	}
	#print "Printing the command.....$command_1"; 
       updatelog("Executing clone command from $SRC to $DEST for printer $migprinters[$j]");	

       #system ($command_1 .">>$LOGFILE") ==0 or die("$datetime: Clone command is unsuccessful.Check log for more information");
       `$command_1` ;
       $ret = $? >> 8;
       if($ret )
       {
		updatelog("Clone command is unsuccessful.Check log for more information");	
		die("$datetime: Clone command is unsuccessful.Check log for more information");	
       }

       $j=$j+1;
}

 
         

if($REMALL){

         print STDOUT "Are you sure you want to delete the old policies[y/n]:";
         my $input = <STDIN>;
         chomp($input);

         if ($input eq "y"){

	 updatelog("The initial policies will get deleted as per your choice");
         print STDOUT "The initial policies are getting deleted.\n";
       
         foreach(@migprinters){
         if ($os =~ /MSWin32/)
	 {
	 $command_3 = "ippman.bat delete -uri \"ipp:\/\/$SRC\/ipp\/$migprinters[$i]\" -server $HOST -port $PORT -username $ADMIN -password $PASSWD";
  	 }    
	 else 
	 {
	 $command_3 = "ippman.sh delete -uri \"ipp:\/\/$SRC\/ipp\/$migprinters[$i]\" -server $HOST -port $PORT -username $ADMIN -password $PASSWD";
  	 }    
	 updatelog("Deleting old printer: $migprinters[$i]");
           
	 #system ($command_3 .">>$LOGFILE") ==0 or die("$datetime: Delete command is unsuccessful. Check the log file for more information");
         `$command_3` ;
         $ret = $? >> 8;
         if($ret )
         {
		updatelog("Delete command is unsuccessful.Check log for more information");	
		die("$datetime: Delete command is unsuccessful.Check log for more information");	
         }
         $i=$i+1;
 	}
}
  else{
	 updatelog("User decided NOT to delete the old policies\n");
         print STDOUT "The initial policies are NOT be deleted\n";

  }

}#end of if($REMALL)

print STDOUT "Successfully updated zen policies\n";        
updatelog("Successfully updated zen policies");
}

sub migrate_all_policies()
{
my $search =$SRC;
my $replace = $DEST;
my $i=0;
if($os =~ /MSWin32/)
{
$tmpfile = 'C:\tmp.xml'; #temp dir can be current directory
}
else
{
$tmpfile = "/tmp/tmp.xml"; #temp dir can be current directory
}
my $file;
my $dir = $XMLDIR;

         foreach(@migprinters){
         if ($os =~ /MSWin32/)
	 {
	 $command_4 = "ippman.bat export -uri \"ipp:\/\/$SRC\/ipp\/$migprinters[$i]\" -folder $dir -server $HOST -port $PORT -username $ADMIN -password $PASSWD";
 	}
	else 
	 {
	 $command_4 = "ippman.sh export -uri \"ipp:\/\/$SRC\/ipp\/$migprinters[$i]\" -folder $dir -server $HOST -port $PORT -username $ADMIN -password $PASSWD";
 	}
	 updatelog("Exporting the migrated printer: $migprinters[$i] to update ");
	 print "Exporting the migrated printer: $migprinters[$i]\n";
	 #system ($command_4 .">>$LOGFILE") ==0 or die("$datetime: ippman Export command is unsuccessful. Check the log file for more information");
         `$command_4` ;
         $ret = $? >> 8;
         if($ret )
         {
		updatelog("ippman Export command is unsuccessful.Check log for more information");	
		die("$datetime: ippman Export command is unsuccessful.Check log for more information");	
         }
         $i=$i+1;
 	}

updatelog("Zen policies are exported to $dir directory");
#change to directory where xml files are present
chdir("$dir") or die "Failed to change directory:$!\n";
updatelog("Changed directory to $dir for xml processing and updating ");
#opendir(DIR,"$dir") or die "can't open $dir \n" ;
opendir(DIR,".") or die "Can not open $dir \n" ;
my @files = sort (grep (/xml$/, readdir DIR)) or die "Cannot open $dir:$! OR $dir directory is empty\n";

updatelog("Reading files: @files");
updatelog("Started processing/updating  Policies");
foreach $file (@files)
{
	chmod(0755,$file);
        open(FILE, "<$file") or  die "Error: The $file can not be opened\n\n";

        open(TMPFILE, " >$tmpfile") or die "Error: The $tmpfile can not be opened\n\n";
        while(<FILE>)
        {
                my $line = $_;
                if($line =~ s/$search/$replace/g )
                {
                        print TMPFILE $line;
                }
                else
                {
                        print TMPFILE $line;
                }
        }
	updatelog("updating policy $file");
	print "updating policy $file\n";

	#rename ($tmpfile,$file);
        #copy ($tmpfile, $file) or die "An error occurred while copying . $! \n" ;
	if($^O =~ /MSWin32/)
	{	
        `copy $tmpfile $file`;
	}
	else
	{
        `cp $tmpfile $file`;
	`rm $tmpfile`;
	}
        #copy ($tmpfile, $file) or die "An error occurred while copying . $! \n" ;
close FILE;
}
chdir("../");
#`cd ../`;
#import all migrated policies to Zen database using ippman.sh command
print "Zen server is updating policies....\n";
if ($^O =~ /MSWin32/)
{
$command_5 = "ippman.bat import -folder $XMLDIR -server $HOST -port $PORT -username $ADMIN -password $PASSWD";
}
else 
{
$command_5 = "ippman.sh import -folder $XMLDIR -server $HOST -port $PORT -username $ADMIN -password $PASSWD";
}
#system ($command_5 .">>$LOGFILE") ==0 or die("$datetime: ippman.sh import command is unsuccessful..Check log for more info");
`$command_5` ;
$ret = $? >> 8;
if($ret )
{
	updatelog("ippman import command is unsuccessful.Check log for more information");	
	die("$datetime: ippman import command is unsuccessful.Check log for more information");	
}
#updatelog("All the policies are migrated successfully\n");
closedir(DIR);
}


sub updatelog($)
{
	unless(-d $LOGDIR)
	{
		mkdir($LOGDIR) or die "$!\n";
	}
	my ($text) = @_;
        open(LOG,">>$LOGFILE")
        or &fileerror($!);
        $datetime = localtime();
        print LOG "$datetime  $text \n"; 
}
  
sub cleanup{
close $LOGFILE; 
}
